/*
 * Copyright 2011 Daniel Josefsson
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package se.dannej.fakehttpserver;


import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.eclipse.jetty.server.Handler;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.AbstractHandler;

import se.dannej.fakehttpserver.expect.Description;
import se.dannej.fakehttpserver.expect.Expectation;
import se.dannej.fakehttpserver.expect.UnexpectedRequestExpectation;

/**
 * {@link FakeServer} that uses Jetty.
 */
public class JettyFakeServer implements FakeHttpServer {
	private final int port;
	private final Server server;
	private final List<Expectation> expectations = new ArrayList<Expectation>();
	private Expectation defaultExpectation = new UnexpectedRequestExpectation();
	private final Handler handler = new AbstractHandler() {
		@Override
		public void handle(String target, Request baseRequest, HttpServletRequest request, HttpServletResponse response)
				throws IOException, ServletException {
			FakeHttpServletRequest fakeHttpServletRequest = new FakeHttpServletRequest(request);
			FakeHttpServletResponse fakeHttpServletResponse = new FakeHttpServletResponse(response);
			
			for (Expectation expectation : expectations) {
				boolean handled = expectation.handle(fakeHttpServletRequest, fakeHttpServletResponse);
				if (handled) {
					return;
				}
			}
			defaultExpectation.handle(fakeHttpServletRequest, fakeHttpServletResponse);
		}
	};

	public JettyFakeServer(int port) {
		this.port = port;
		server = new Server(port);
		server.setHandler(handler);
	}

	/**
	 * Set the default handler, invoked a request does not match any expectations.
	 * @param defaultExpectation expectation to invoke for unexpected request
	 */
	public void setDefaultExpectation(Expectation defaultExpectation) {
		this.defaultExpectation = defaultExpectation;
	}

	/**
	 * Add an expectation (a request handler) to the server.
	 * 
	 * @param expectation
	 */
	@Override
	public void addExpectation(final Expectation expectation) {
		expectations.add(expectation);
	}

	@Override
	public void start() {
		try {
			server.start();
		} catch (Exception e) {
			throw new FakeServerException("Failed starting jetty", e);
		}
	}

	@Override
	public void stop() {
		if (server.isRunning()) {
			try {
				server.stop();
			} catch (Exception e) {
				throw new FakeServerException("Failed stopping jetty", e);
			}
		}
	}

	@Override
	public void assertIsSatisfied() {
		Description description = new Description();
		description.appendText("jetty-server:<" + port + "> ");
		
		for (Expectation expectation : expectations) {
			expectation.assertIsSatisfied(description);
		}
		
		defaultExpectation.assertIsSatisfied(description);
	}
}
