/*
 * Copyright 2011 Daniel Josefsson
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package se.dannej.fakehttpserver.expect.matcher;

import java.util.Enumeration;

import org.hamcrest.Matcher;

import se.dannej.fakehttpserver.FakeHttpServletRequest;
import se.dannej.fakehttpserver.expect.Description;

/**
 * Class containing matchers for headers.
 */
public class HeaderMatchers {

	/**
	 * @param name header name
	 * @param value header value
	 * @return a header matcher, ignoring case on header name.
	 */
	public static RequestMatcher header(final String name, final String value) {
		return new RequestMatcher() {
			@Override
			public boolean matches(FakeHttpServletRequest request) {
				Enumeration<String> names = request.getHeaders(name);
				while (names.hasMoreElements()) {
					if (value.equals(names.nextElement())) {
						return true;
					}
				}
				return false;
			}
			
			@Override
			public Description getDescription() {
				Description description = new Description();
				description.appendText(String.format("header [%s,%s]", name, value));
				return description;
			}
		};
	}

	/**
	 * @param nameMatcher header name matcher
	 * @param valueMatcher header value matcher
	 * @return a header matcher
	 */
	public static RequestMatcher header(final Matcher<String> nameMatcher, final Matcher<String> valueMatcher) {
		return new RequestMatcher() {
			@Override
			public boolean matches(FakeHttpServletRequest request) {
				Enumeration<String> names = request.getHeaderNames();
				while (names.hasMoreElements()) {
					String name = names.nextElement();
					if (nameMatcher.matches(name)) {
						Enumeration<String> values = request.getHeaders(name);
						while (values.hasMoreElements()) {
							String value = values.nextElement();
							if (valueMatcher.matches(value)) {
								return true;
							}
						}
					}
				}
				return false;
			}
			
			@Override
			public Description getDescription() {
				Description description = new Description();
				description.appendText(String.format("header [%s,%s]", nameMatcher.toString(), valueMatcher.toString()));
				return description;
			}
		};
	}

}
