/*
 * Copyright 2011 Daniel Josefsson
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package se.dannej.fakehttpserver.junit;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import org.junit.rules.TestRule;
import org.junit.runners.BlockJUnit4ClassRunner;
import org.junit.runners.model.InitializationError;

import se.dannej.fakehttpserver.FakeServer;
import se.dannej.fakehttpserver.FakeServerException;

/**
 * A Test runner that decorates a junit test with a {@link FakeServerRule}.
 * It introspects the test class and finds the fake server fields, in order
 * to shut them down after the test.
 */
public class FakeServerRunner extends BlockJUnit4ClassRunner {

	public FakeServerRunner(Class<?> testClass) throws InitializationError {
		super(testClass);
	}

	@Override
	protected List<TestRule> getTestRules(Object target) {
		List<TestRule> testRules = new ArrayList<TestRule>();
		testRules.addAll(super.getTestRules(target));
		testRules.add(new FakeServerRule(locateFakeServers(target)));
		return testRules;
	}

	private List<FakeServer> locateFakeServers(Object target) {
		return recursivelyLocateFieldsOfClass(target, target.getClass(), FakeServer.class);
	}

	private static <T> List<T> recursivelyLocateFieldsOfClass(Object target, Class<? extends Object> targetClass, Class<T> classOfField) {
		List<T> result = new ArrayList<T>();
		
		Class<? extends Object> superClass = targetClass.getSuperclass();
		if (superClass != null) {
			result.addAll(recursivelyLocateFieldsOfClass(target, superClass, classOfField));
		}
		
		result.addAll(locateFieldsOfClass(target, targetClass, classOfField));
		
		return result;
	}

	@SuppressWarnings("unchecked")
	private static <T> List<T> locateFieldsOfClass(Object target, Class<? extends Object> targetClass,
			Class<T> classOfField) {
		List<T> result = new ArrayList<T>();
		Field[] declaredFields = targetClass.getDeclaredFields();
		for (Field field : declaredFields) {
			field.setAccessible(true);
			if (classOfField.isAssignableFrom(field.getType())) {
				try {
					T found = (T) field.get(target);
					result.add(found);
				} catch (Exception e) {
					throw new FakeServerException("Failed value of field " + field.getName() + " from " + target, e);
				}
			}
		}
		return result;
	}
}
